package ucl.salinization.context;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import cern.jet.random.Normal;

import repast.simphony.context.Context;
import repast.simphony.context.space.gis.GeographyFactoryFinder;
import repast.simphony.context.space.graph.NetworkFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.GUIRegistryType;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.gis.Geography;
import repast.simphony.space.gis.GeographyParameters;
import repast.simphony.space.graph.Network;

import ucl.salinization.data.ScenarioData;
import ucl.salinization.dataLoad.ScenarioXMLFileLoader;
import ucl.salinization.displays.DisplayCreator;
import ucl.salinization.model.physical.FieldSetting;
import ucl.salinization.model.physical.HydrologicalSetting;
import ucl.salinization.model.physical.HydrologicalUnit;
import ucl.salinization.model.physical.SalinizationModel;
import ucl.salinization.social.Community;

/**
 * Simulation context builder.
 * 
 * @author 
 *
 * @param <T>
 */
public class SalinizationContextBuilder<T> implements ContextBuilder<T>{

//	private Network<T> populationNetwork;
	private Network<T> hydrologyNetwork;
	private Geography geography;
	private static boolean started=false;
	
	@Override
	public Context build(Context<T> context) {
		if(started && !RunEnvironment.getInstance().isBatch()){
				started=false;
				return null;
		}
		System.out.println("Running salinization...");
		  Parameters p = RunEnvironment.getInstance().getParameters();
		  String scenarioData=p.getValueAsString("scenarioData");
		
//		 populationNetwork=NetworkFactoryFinder.createNetworkFactory(null).
//	      	createNetwork("Population", context, true);
		 
		  hydrologyNetwork=NetworkFactoryFinder.createNetworkFactory(null).
	      	createNetwork("Hydrology", context, true);
		
		  GeographyParameters geoParams = new GeographyParameters();
		  geography = GeographyFactoryFinder.createGeographyFactory(null)
       	   .createGeography("Geography", context, geoParams);
		
		  int numberYears=(Integer)p.getValue("numberOfYears");
		  double daysPerTick=(Double)p.getValue("daysPerTick");
		  double endTick=(365.25/daysPerTick)*numberYears;
		 
		  RunEnvironment.getInstance().endAt(Double.valueOf(endTick));
		 
		  //build from files
		  buildFromFiles(context,scenarioData);
		  
		  buildSalinizationModel(p,context);
		  
		  if (!RunEnvironment.getInstance().isBatch()){
			  started=true;
	//		  List<String> shapeFileList = new ArrayList<String>();
	//			shapeFileList.add(file);
	//			MSCMapDisplay.init(popsO, new AirspaceLoaderGame6(), shapeFileList );
				
				 // Add the display to the GUI registry.
	//			RunState.getInstance().getGUIRegistry().addDisplay("Tree",
	//	    		GUIRegistryType.DISPLAY, 	new DisplayCreator().createDisplay(context, 
	//	    				DisplayCreator.DisplayType.STEP));
		  }
		  return context;
	}
	
	/**
	 * Method to build the salinization model.
	 * @param p
	 * @param context
	 */
	public void buildSalinizationModel(Parameters p,Context context){
		 double panCoeff=(Double)p.getValue("panEvaporation");
		 double stdPanCoeff=(Double)p.getValue("panEvaporationSD");
		 double empCoeff=(Double)p.getValue("empiricalCoefficient");
		 double rainfallSal=(Double)p.getValue("rainfallSalinity");
		 double irrigationSal=(Double)p.getValue("irrigationSalinity");
		 double saltT=Double.parseDouble((String)p.getValue("saltTolerance"));
		 double fallowS=Double.parseDouble((String)p.getValue("fallowScaling"));
		 
		 SalinizationModel sm = new SalinizationModel();
		 
		 Normal normal=RandomHelper.createNormal(panCoeff, stdPanCoeff);
		 sm.setNormalPanEvaporation(normal);
		 sm.setEmpiricalCoefficient(empCoeff);
		 sm.setRainfallSalinity(rainfallSal);
		 sm.setIrrigationSalinity(irrigationSal);
		 sm.setSaltTolerance(saltT);
		 sm.setFallowScaling(fallowS);
		 
		 context.add(sm);
	}
	
	/**
	 * Method to get input files referenced by a scenario file
	 * @param context the master context
	 * @param fileData the path to the scenario file
	 */
	public void buildFromFiles(Context context, String file){
		ScenarioXMLFileLoader sxf=ScenarioXMLFileLoader.SetupScenarioFileData(file);
		Iterator<ScenarioData> ic = sxf.getPopulations().iterator();
		ScenarioBuilder sc = new ScenarioBuilder();
		
		List<HydrologicalUnit>hydroUnits = new ArrayList<HydrologicalUnit>();
		while(ic.hasNext()){
			ScenarioData sd = ic.next();
					
			if(sd.getCommunityFile()!=null){   //build a population and water use model
				List<Community>communities=sc.buildPopulation(context,sd,geography);
				Iterator<Community> ico = communities.iterator();
				while(ico.hasNext()) {
					Community c = ico.next();
					context.add(c);
				}
			}
			
			if(sd.getStreamFlowFile()!=null || sd.getHydrologicalFile()!=null) {    //any scenario with interactive hydrology
				HydrologicalSetting hs=sc.buildHydrology(context, geography,sd,sd.getHydrologyModel());
				context.addSubContext(hs);
				context.add(hs);
			}
			FieldSetting fs=sc.buildFields(context,geography,sd);
	//		FieldSetting fs=sc.buildFieldColors(context, geography, sd);
			sc.createRainfallData(sd.getRainfallFile());
//			fs.setOutput(outputContext);
			context.addSubContext(fs);
			context.add(fs);
		}
	}

}
