package ucl.salinization.model.physical;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.space.gis.Geography;
import repast.simphony.visualization.IDisplay;
import repast.simphony.visualization.gis3D.DisplayGIS3D;
import repast.simphony.visualization.gisnew.AbstractSurfaceLayer;
import repast.simphony.visualization.gisnew.NewDisplayGIS;
import ucl.salinization.displays.DefaultStyleGIS;
import ucl.salinization.displays.GShape;
import ucl.salinization.output.Outputter;
import ucl.salinization.social.Community;
import ucl.salinization.social.Person;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Envelope;


import gov.nasa.worldwind.awt.WorldWindowGLCanvas;
import gov.nasa.worldwind.geom.LatLon;
import gov.nasa.worldwind.layers.RenderableLayer;
import gov.nasa.worldwind.render.Material;
import gov.nasa.worldwind.render.SurfacePolygon;
import gov.nasa.worldwind.render.airspaces.Polygon;

public class Field {

	public static int FALLOW_FIELD=0;
	public static int PREPLANTING=1;
	public static int PLANTED=2;
	public static int HARVESTED=3;
	public static int IRRIGATION=4;
	
	public boolean FALLOW=true;
	public int plantYear=0;
	
	Person person;
	public String name="";
	public double leachingEfficiency=1.0;
	public static int ALLUVIAL=1;
	public static int GENERAL=0;
	public static int HIGH_DRAIN=-1;
	public Polygon polygon;
	public Crop crop=Crop.BARLEY;
	private static int ID=0;
	private boolean fallow=false;
	private double fallowYears=0.0;
	private double nitrogen=1.0;
	private double potasium=1.0;
	private double yieldMultiplier=1.0;
	private double yield;
	private double saltCompounds=0.01;
	private double trueFallowYears=0d;
	private int id=-1;
	private int status;
	private int fieldType=0;
	private SurfacePolygon surfacePolygon;
	private boolean fertUpdate=false;
	private Point centroid;
	private Community community;
	private int year=0;
	private MultiPolygon poly;
	private HydrologicalUnit fieldWater;
	private double cropCoeff=0d;
	private double stdCropCoeff=0d;
	private double capillarySalt=0d;
	private double capillarySaltSD=0d;
	private double saltRemovePercent=0d;
	private double leachingFactor=0;
	private NewDisplayGIS gis;
	private double belowRootZoneSalt=0;
	private double waterTableSalt=0.001;  //in ds/m -1
	private double ratioUptake=1.0;
	private double layerThickness=0.0;
	private double initialSalinization=0.0; //salt initially in the root zone
	private double appliedWaterSalinity=0.0;
	private double soilPercolationRate=0.0;  //soil percolation rate of salt
	private double waterStoredSaturation=0.0; //water stored in the soil at field saturation
	private double soilWaterSalinity=0.0;  //salinity of soil water
	private double percolationRate=1.0;  //m per year
	private double sRemove=0;
	private double leachingSTD;
	private double initialLeachingFactor;
	private double initialStandardDeviation;
	private double initialEfficiency;
	private double initialRatioUptake;
	private double initialWaterTable;
	
	public Field(MultiPolygon poly){
		this.poly=poly;
		Coordinate[] points = poly.getCoordinates();
		this.centroid=poly.getCentroid();
		List<LatLon> pPoints = new ArrayList<LatLon>();
		for(int i=0; i < points.length; i++){
			Coordinate c = points[i];
			LatLon point=LatLon.fromDegrees(c.y, c.x);
			pPoints.add(point);
		}
//		this.id=ID;
		surfacePolygon=new SurfacePolygon(pPoints);
//		ID++;
		this.cropCoeff=(Double)RunEnvironment.getInstance().getParameters().getValue("cropCoefficient");
		this.stdCropCoeff=(Double)RunEnvironment.getInstance().getParameters().getValue("cropCoefficientSD");
	}
	
	
	
	public double getTrueFallowYears() {
		return trueFallowYears;
	}



	public void setTrueFallowYears(double trueFallowYears) {
		this.trueFallowYears = trueFallowYears;
	}



	public Crop getCrop() {
		return crop;
	}


	public void setCrop(Crop crop) {
		this.crop = crop;
	}


	public MultiPolygon getPoly() {
		return poly;
	}

	public void setPoly(MultiPolygon poly) {
		this.poly = poly;
	}
	
	@ScheduledMethod(start = 0, pick=ScheduledMethod.ALL, priority = Double.MAX_VALUE)
	public void setupViz(){
		List<IDisplay> displays=RunState.getInstance().getGUIRegistry().getDisplays();
		Iterator<IDisplay> ic = displays.iterator();
		this.gis=null;
		while(ic.hasNext()){
			IDisplay display = ic.next();
			if(display instanceof NewDisplayGIS)
				gis=(NewDisplayGIS)display;
		}
	/*	WorldWindowGLCanvas canvas=gis.getWwglCanvas();
		RenderableLayer rl = new enderableLayer();
		rl.addRenderable(surfacePolygon);
		canvas.getModel().getLayers().add(rl); */
//		GShape gs=(GShape)gis.getClassStyleMap().get(Field.class).getVisualItem(this);
//		gs.updateRederable(gis,this);
//		gis.getWwglCanvas().redrawNow();
		findWaterConnection();
	}
	
/*	public void updateField(){
		GShape gs=(GShape)gis.getClassStyleMap().get(Field.class).getVisualItem(this);
		gs.updateRederable(gis,this);
	}*/
	
	/**
	 * Method for finding the water system associated with the field.
	 */
	public void findWaterConnection(){
		Iterator<HydrologicalUnit> hi=RunState.getInstance().getMasterContext().
				getAgentLayer(HydrologicalUnit.class).iterator();
			
			double dist=0;
			HydrologicalUnit keepUnit=null;
			while(hi.hasNext()){
				HydrologicalUnit hu = hi.next();
				double d=this.centroid.distance(hu.getGeometry().getCentroid());
				
				if(dist==0 || d<dist){
					dist=d;
					keepUnit=hu;
					continue;
				}
			}
			if(keepUnit!=null)
				this.fieldWater=keepUnit;
			
	}
	
	public double getNitrogen() {
		return nitrogen;
	}
	public void setNitrogen(double nitrogen) {
		this.nitrogen = nitrogen;
	}
	public double getPotasium() {
		return potasium;
	}
	public void setPotasium(double potasium) {
		this.potasium = potasium;
	}
	public double getYield() {
		return yield;
	}
	public void setYield(double yield) {
		this.yield = yield;
	}
	public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}

	public int getStatus() {
		return status;
	}

	public void setStatus(int status) {
		this.status = status;
	}

	public double getYieldMultiplier() {
		return yieldMultiplier;
	}

	public void setYieldMultiplier(double yieldMultiplier) {
		this.yieldMultiplier = yieldMultiplier;
	}

	public int getFieldType() {
		return fieldType;
	}

	public void setFieldType(int fieldType) {
		this.fieldType = fieldType;
	}

	public boolean isFertUpdate() {
		return fertUpdate;
	}

	public void setFertUpdate(boolean fertUpdate) {
		this.fertUpdate = fertUpdate;
	}

	public Point getCentroid() {
		return centroid;
	}

	public void setCentroid(Point centroid) {
		this.centroid = centroid;
	}

	public int getYear() {
		return year;
	}

	public void setYear(int year) {
		this.year = year;
	}

	public boolean isFallow() {
		return fallow;
	}

	public void setFallow(boolean fallow) {
		this.fallow = fallow;
	}

	public SurfacePolygon getSurfacePolygon() {
		return surfacePolygon;
	}

	public void setSurfacePolygon(SurfacePolygon surfacePolygon) {
		this.surfacePolygon = surfacePolygon;
	}

	public Community getCommunity() {
		return community;
	}

	public void setCommunity(Community community) {
		this.community = community;
	}

	public double getSaltCompounds() {
		return saltCompounds;
	}

	public void setSaltCompounds(double saltCompounds) {
		this.saltCompounds = saltCompounds;
	}


	public int getPlantYear() {
		return plantYear;
	}


	public void setPlantYear(int plantYear) {
		this.plantYear = plantYear;
	}


	public double getLeachingFactor() {
		return leachingFactor;
	}


	public void setLeachingFactor(double leachingFactor) {
		this.leachingFactor = leachingFactor;
	}


	public Person getPerson() {
		return person;
	}


	public void setPerson(Person person) {
		this.person = person;
	}


	public HydrologicalUnit getFieldWater() {
		return fieldWater;
	}


	public void setFieldWater(HydrologicalUnit fieldWater) {
		this.fieldWater = fieldWater;
	}


	public double getCropCoeff() {
		return cropCoeff;
	}


	public void setCropCoeff(double cropCoeff) {
		this.cropCoeff = cropCoeff;
	}


	public double getStdCropCoeff() {
		return stdCropCoeff;
	}


	public void setStdCropCoeff(double stdCropCoeff) {
		this.stdCropCoeff = stdCropCoeff;
	}

	public double getCapillarySalt() {
		return capillarySalt;
	}

	public void setCapillarySalt(double capillarySalt) {
		this.capillarySalt = capillarySalt;
	}

	public double getCapillarySaltSD() {
		return capillarySaltSD;
	}

	public void setCapillarySaltSD(double capillarySaltSD) {
		this.capillarySaltSD = capillarySaltSD;
	}

	public double getSaltRemovePercent() {
		return saltRemovePercent;
	}

	public void setSaltRemovePercent(double saltRemoveP) {
		this.saltRemovePercent = saltRemoveP;
	}

	public Polygon getPolygon() {
		return polygon;
	}

	public void setPolygon(Polygon polygon) {
		this.polygon = polygon;
	}

	public double getBelowRootZoneSalt() {
		return belowRootZoneSalt;
	}

	public void setBelowRootZoneSalt(double belowRootZoneSalt) {
		this.belowRootZoneSalt = belowRootZoneSalt;
	}

	public double getFallowYears() {
		return fallowYears;
	}

	public void setFallowYears(double fallowYears) {
		this.fallowYears = fallowYears;
	}



	public double getWaterTableSalt() {
		return waterTableSalt;
	}



	public void setWaterTableSalt(double waterTableSalt) {
		this.waterTableSalt = waterTableSalt;
	}



	public double getRatioUptake() {
		return ratioUptake;
	}



	public void setRatioUptake(double ratioUptake) {
		this.ratioUptake = ratioUptake;
	}



	public double getLayerThickness() {
		return layerThickness;
	}



	public void setLayerThickness(double layerThickness) {
		this.layerThickness = layerThickness;
	}



	public double getInitialSalinization() {
		return initialSalinization;
	}



	public void setInitialSalinization(double initialSalinization) {
		this.initialSalinization = initialSalinization;
	}



	public double getAppliedWaterSalinity() {
		return appliedWaterSalinity;
	}



	public void setAppliedWaterSalinity(double appliedWaterSalinity) {
		this.appliedWaterSalinity = appliedWaterSalinity;
	}



	public double getSoilPercolationRate() {
		return soilPercolationRate;
	}



	public void setSoilPercolationRate(double soilPercolationRate) {
		this.soilPercolationRate = soilPercolationRate;
	}



	public double getLeachingEfficiency() {
		return leachingEfficiency;
	}



	public void setLeachingEfficiency(double leachingEfficiency) {
		this.leachingEfficiency = leachingEfficiency;
	}



	public double getWaterStoredSaturation() {
		return waterStoredSaturation;
	}



	public void setWaterStoredSaturation(double waterStoredSaturation) {
		this.waterStoredSaturation = waterStoredSaturation;
	}



	public double getSoilWaterSalinity() {
		return soilWaterSalinity;
	}



	public void setSoilWaterSalinity(double soilWaterSalinity) {
		this.soilWaterSalinity = soilWaterSalinity;
	}



	public double getPercolationRate() {
		return percolationRate;
	}



	public void setPercolationRate(double percolationRate) {
		this.percolationRate = percolationRate;
	}



	public double getsRemove() {
		return sRemove;
	}



	public void setsRemove(double sRemove) {
		this.sRemove = sRemove;
	}



	public double getLeachingSTD() {
		return leachingSTD;
	}



	public void setLeachingSTD(double leachingSTD) {
		this.leachingSTD = leachingSTD;
	}



	public String getName() {
		return name;
	}



	public void setName(String name) {
		this.name = name;
	}



	public double getInitialLeachingFactor() {
		return initialLeachingFactor;
	}



	public void setInitialLeachingFactor(double initialLeachingFactor) {
		this.initialLeachingFactor = initialLeachingFactor;
	}



	public double getInitialStandardDeviation() {
		return initialStandardDeviation;
	}



	public void setInitialStandardDeviation(double initialStandardDeviation) {
		this.initialStandardDeviation = initialStandardDeviation;
	}



	public double getInitialEfficiency() {
		return initialEfficiency;
	}



	public void setInitialEfficiency(double initialEfficiency) {
		this.initialEfficiency = initialEfficiency;
	}



	public double getInitialRatioUptake() {
		return initialRatioUptake;
	}



	public void setInitialRatioUptake(double initialRatioUptake) {
		this.initialRatioUptake = initialRatioUptake;
	}



	public double getInitialWaterTable() {
		return initialWaterTable;
	}



	public void setInitialWaterTable(double initialWaterTable) {
		this.initialWaterTable = initialWaterTable;
	}
	
	
	
}
