package ucl.salinization.model.physical;

import gov.nasa.worldwind.geom.LatLon;
import gov.nasa.worldwind.geom.Position;
import gov.nasa.worldwind.layers.Layer;
import gov.nasa.worldwind.layers.RenderableLayer;
import gov.nasa.worldwind.render.ExtrudedPolygon;
import gov.nasa.worldwind.render.Material;
import gov.nasa.worldwind.render.Renderable;
import gov.nasa.worldwind.render.ShapeAttributes;
import gov.nasa.worldwind.render.SurfacePolygon;
import gov.nasa.worldwind.render.airspaces.AirspaceAttributes;
import gov.nasa.worldwind.render.airspaces.BasicAirspaceAttributes;
import gov.nasa.worldwind.render.airspaces.Polygon;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jscience.geography.coordinates.LatLong;

import repast.simphony.context.DefaultContext;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.IAction;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.random.RandomHelper;
import repast.simphony.visualization.IDisplay;
import repast.simphony.visualization.gisnew.NewDisplayGIS;
import repast.simphony.visualization.gisnew.StyleGIS;
import repast.simphony.visualization.gisnew.StyledSurfaceShapeLayer;

import ucl.salinization.displays.DefaultStyleGIS;
import ucl.salinization.displays.GShape;
import ucl.salinization.output.FieldOutput;
import ucl.salinization.output.Outputter;

public class FieldSetting extends DefaultContext<Field>{

	List<Field>fields;
	RenderableLayer renderLayer;
	private NewDisplayGIS gis;
	int one=0;
	private DefaultStyleGIS style;
	public FieldSetting(){
		super("fieldSetting","Field Setting");
		
	}
	
	private class FieldWritter implements IAction{
		public void execute(){
			writeCSVOutput();
		}
	}
	
	private class ScheduleFields implements IAction{
		public void execute() {
			writeShapefileFieldOutput();
		}
	}
	
	private class ScheduleRepeatsFields implements IAction{
		public void execute() {
			writeShapefileRepeat();
		}
	}
	
	public void writeCSVOutput(){
		double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		Outputter.writeFileData(tick, fields);
	}
	
//	@ScheduledMethod(start = 12, interval=12,pick=ScheduledMethod.ALL, priority = Double.MAX_VALUE)
	public void updateFields(){
		
		for(Field f: fields){
			f.getPolygon().getAttributes().setMaterial(new Material(this.style.getFillColor(f)));
		}
	}
	
	@ScheduledMethod(start = 0,interval=100,pick=ScheduledMethod.ALL, priority = Double.MAX_VALUE)
	public void startFields(){
		
		if(this.gis==null){
			List<IDisplay> displays=RunState.getInstance().getGUIRegistry().getDisplays();
			Iterator<IDisplay> ic = displays.iterator();
			this.gis=null;
			while(ic.hasNext()){
				IDisplay display = ic.next();
				if(display instanceof NewDisplayGIS)
					gis=(NewDisplayGIS)display;
			}
		}
		
	/*	if(RunEnvironment.getInstance().getCurrentSchedule().getTickCount()==1.0){
		for(Field f:fields){
				GShape gs=(GShape)gis.getClassStyleMap().get(Field.class).getVisualItem(f);	
				if(gs.updateRederable(gis,f)){
					break;
				}
			}
		}*/
		
		addLayer(fields,gis);
		
		
	}
	
	public void addLayer(List<Field>fields,NewDisplayGIS gis){
		if(RunState.getInstance().getRunInfo().isBatch())
			return;
//		if(gis.getWwglCanvas().getModel().getLayers().contains(this.renderLayer))
//			gis.getWwglCanvas().getModel().getLayers().remove(this.renderLayer);
		ArrayList<Renderable> renders=new ArrayList<Renderable>();
		for(Field f: fields){
				SurfacePolygon sp = f.getSurfacePolygon();
				
				
				BasicAirspaceAttributes ab=new BasicAirspaceAttributes();
			
				Iterator<? extends LatLon> ic = sp.getLocations().iterator();
				ArrayList<Position>positions=new ArrayList<Position>();
				double ele=0;
				while(ic.hasNext()){
					LatLon l = ic.next();
					double elevation=10+gis.getWwglCanvas().getModel().getGlobe().getElevation(l.getLatitude(),
							l.getLongitude());
					Position pp = new Position(l,elevation);
					ele=elevation;
					positions.add(pp);
				} 
				Polygon p = new Polygon(positions);
	//			p.setTerrainConforming(true);
				this.style=new DefaultStyleGIS();
				f.setPolygon(p);
				
				ShapeAttributes attrs = sp.getAttributes();
				ab.setMaterial(new Material(style.getFillColor(f)));
				ab.setOpacity(0.5);
				ab.setDrawInterior(true);
	//			ab.setOutlineOpacity(1.0);
	//			ab.setOutlineMaterial(new Material(style.getBorderColor(f)));
	//			attrs.setInteriorOpacity(style.getFillOpacity(f));
	//			attrs.setInteriorMaterial(new Material(style.getFillColor(f)));
				attrs.setOutlineMaterial(new Material(style.getBorderColor(f)));
				attrs.setOutlineOpacity(style.getBorderOpacity(f));
				attrs.setOutlineWidth(3);
				p.setAttributes(ab);
				sp.setAttributes(attrs);
				p.setAltitude(ele+100);
				RenderableLayer rl = new RenderableLayer();
				rl.addRenderable(p);
				
				gis.getWwglCanvas().getModel().getLayers().add(rl);
				
	//		}
	//		else{
	//			f.getPolygon().getAttributes().setMaterial(new Material(Color.black));
	//		}
		}
/*		if(one==0){
			if(this.renderLayer==null)
				this.renderLayer = new RenderableLayer();
			if(!gis.getWwglCanvas().getModel().getLayers().contains(this.renderLayer)){
				renderLayer.addRenderables(renders);
				gis.getWwglCanvas().getModel().getLayers().add(renderLayer);
			}
		}*/
		HydrologicalSetting hs=
			(HydrologicalSetting) RunState.getInstance().getMasterContext().getAgentLayer(HydrologicalSetting.class).iterator().next();
		hs.render();
		one++;
//		gis.render();
	}
	
	@ScheduledMethod(start = 0, pick=ScheduledMethod.ALL, priority = 0)
	public void runModel(){
//		if(model.getOutput()==null)
//			model.setOutput(output);
//		model.run();
		Outputter.clearFile();
		//schedule outputs, first is the final output
		ISchedule schedule3 = repast.simphony.engine.environment.RunEnvironment.getInstance()
        .getCurrentSchedule();
		  
		  schedule3.schedule(ScheduleParameters.createAtEnd(ScheduleParameters.LAST_PRIORITY),
            new ScheduleFields());
		  
//		  double daysPT=(Double)RunEnvironment.getInstance().getParameters().getValue("daysPerTick");
//		  double tick=365.25/daysPT;
//		  double startTick=213/daysPT;
		  
//		  ISchedule schedule5 = repast.simphony.engine.environment.RunEnvironment.getInstance()
//	        .getCurrentSchedule();
			  
//			  schedule5.schedule(ScheduleParameters.createRepeating(startTick,tick,ScheduleParameters.LAST_PRIORITY),
//	            new FieldWritter());
		  
		  //this schedules periodic outputs for fields
		  String times=(String)RunEnvironment.getInstance().getParameters().getValue("fieldTimes");
		  String[] ts = times.split(",");
		  for(String s: ts){
			  ISchedule schedule4 = repast.simphony.engine.environment.RunEnvironment.getInstance()
		        .getCurrentSchedule();
				  
				  schedule4.schedule(ScheduleParameters.createOneTime(Double.parseDouble(s),ScheduleParameters.LAST_PRIORITY),
		            new ScheduleRepeatsFields());
		  }
	}

	public List<Field> getFields() {
		return fields;
	}

	public void setFields(List<Field> fields) {
		this.fields = fields;
	}

	/**
	 * Method for outputting the end of the simulation output.
	 */
	public void writeShapefileFieldOutput(){
		try {
			FieldOutput.writeShapefile(fields, "fields");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * Method for outputting periodic shapefile outputs.
	 */
	public void writeShapefileRepeat(){
		double tick=RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		try {
			FieldOutput.writeShapefile(fields, "fields"+Math.rint(tick));
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
}