package ucl.salinization.model.physical;


import ucl.salinization.util.SpatialHydrologyCalculations;
import gov.nasa.worldwind.render.Renderable;

import com.vividsolutions.jts.geom.Geometry;

/**
 * Class representing any physical hydrological system
 * @author
 */
public class HydrologicalUnit {

	private Geometry geometry; //geometric area of the unit
	private int identification;
	private double averageMeanderLength;  //length of a meander along the meander (i.e measured from one end to the other of a loop)
	private double slope;  //can be topographic slope over a given distance or measured channel slope can be used
	private double waterWidth;  //width of a hydroloigcal unit based on water present
	private double bankfullWidth;  //the full extend of a channel from bank to bank
	private double discharge;  //volume over time
	private double recharge;  //rate of volume that fills in over time
	private double volume;
	private double quality;
	private double amplitude;  //amplitude used for varying volume/discharge metrics 
	private double distanceToTown;
	private int timesVisited;
	private boolean isMunicipalSource=false;
	private boolean evolvedSignificantly=false;
	private String name;
	private Renderable renderable;
	
	public HydrologicalUnit(Geometry geometry, Renderable renderable){
		this.quality=1d;
		this.geometry=geometry;
		this.volume=0;
		this.discharge=0;
		this.amplitude=.05;
		this.identification=-1;
		this.renderable=renderable;
		this.timesVisited=0;
	}
	
	public Geometry getGeometry() {
		return geometry;
	}
	public void setGeometry(Geometry geometry) {
		this.geometry = geometry;
	}
	public double getDischarge() {
		return discharge;
	}
	public void setDischarge(double discharge) {
		this.discharge = discharge;
	}
	public double getRecharge() {
		return recharge;
	}
	public void setRecharge(double recharge) {
		this.recharge = recharge;
	}
	public double getVolume() {
		return volume;
	}
	public void setVolume(double volume) {
		this.volume = volume;
	}
	public int getIdentification() {
		return identification;
	}
	public void setIdentification(int id) {
		this.identification = id;
	}
	public double getQuality() {
		return quality;
	}
	public void setQuality(double quality) {
		this.quality = quality;
	}
	public double getAmplitude() {
		return amplitude;
	}
	public void setAmplitude(double amplitude) {
		this.amplitude = amplitude;
	}
	public Renderable getRenderable() {
		return renderable;
	}
	public void setRenderable(Renderable shape) {
		this.renderable = shape;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}

	public boolean isMunicipalSource() {
		return isMunicipalSource;
	}

	public void setMunicipalSource(boolean isMunicipalSource) {
		this.isMunicipalSource = isMunicipalSource;
	}

	public double getAverageMeanderLength() {
		return averageMeanderLength;
	}

	public void setAverageMeanderLength(double averageMeanderLength) {
		this.averageMeanderLength = averageMeanderLength;
	}

	public double getSlope() {
		return slope;
	}

	public void setSlope(double slope) {
		this.slope = slope;
	}

	public double getWaterWidth() {
		return waterWidth;
	}

	public void setWaterWidth(double channelWidth) {
		this.waterWidth = channelWidth;
	}

	public double getBankfullWidth() {
		return bankfullWidth;
	}

	public void setBankfullWidth(double bankfullWidth) {
		this.bankfullWidth = bankfullWidth;
	}
	
	/**
	 * Method run in order to find daily volume if you have discharge only
	 * @return volume based on discharge rate
	 */
	public double calculateDailyVolumeFromDischarge(){
		this.setVolume(SpatialHydrologyCalculations.DailyVolumeCalculator(this.getDischarge()));
		return volume;
	}

	public double getDistanceToTown() {
		return distanceToTown;
	}

	public void setDistanceToTown(double distanceToTown) {
		this.distanceToTown = distanceToTown;
	}

	public int getTimesVisited() {
		return timesVisited;
	}

	public void setTimesVisited(int timesVisited) {
		this.timesVisited = timesVisited;
	}
	
	public void setHasEvolvedSignificantly(boolean evolved){
		this.evolvedSignificantly=evolved;
	}
	
	public boolean hasEvolvedSignificantly(){
		return evolvedSignificantly;
	}
	
}
