package ucl.salinization.social;

import gov.nasa.worldwind.geom.Angle;
import gov.nasa.worldwind.geom.LatLon;
import gov.nasa.worldwind.geom.Position;
import gov.nasa.worldwind.layers.IconLayer;
import gov.nasa.worldwind.render.Renderable;
import gov.nasa.worldwind.render.UserFacingIcon;

import java.awt.Dimension;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.space.gis.Geography;
import repast.simphony.visualization.IDisplay;
import repast.simphony.visualization.gis3D.DisplayGIS3D;
import repast.simphony.visualization.gis3D.StyledMapLayer3D;
import repast.simphony.visualization.gisnew.AbstractSurfaceLayer;
import repast.simphony.visualization.gisnew.NewDisplayGIS;
import ucl.salinization.model.landuse.LandUse;
import ucl.salinization.model.physical.Field;
import ucl.salinization.output.Outputter;

import com.vividsolutions.jts.geom.Point;


/**
 * Class for the person agent in the simulation
 * @author
 */
public class Person {

	private boolean initV=true;
	private Point location; //geometry of home location
	private boolean alive;
	private int age;
	private Sex sex;
	public static int Id=0;
	private int id;
	private Community community;  //person's community
	private double quantityBelief;  //belief of water quantity
	private double quantityChangeBelief;  //change belief value
	private double qualityBelief;
	private double qualityChangeBelief;
	private UserFacingIcon icon;  //person icon
	private double sourcePreference;  //prefernce for a water source
	private int sourceVisited;  //current source visited by agent
	private double otherSourceQuantityBelief;
	private double otherSourceQuantityChangeBelief;
	private double otherSourceQualityChangeBelief;
	private double otherSourceQualityBelief;
	private double motivation; //motivation to accomplish tasks or enhance resilience
	private boolean accuratePerception;
	private double waterConsumed;  //water consumed at given time
	private double importance; //how much an agent affects operations of social behavior
	private double decisionCost; //variable for cost of decision making
	private double x;  //arbitrary x location
	private double y; //arbitrary y location
	private int agree=0; //agree on something or not
	private Field field; // field cropped by farmer/person
	private LandUse landUse; //land use model
	private NewDisplayGIS gis;
	private IconLayer layer2;
	
	public double getDecisionCost() {
		return decisionCost;
	}

	public void setDecisionCost(double decisionCost) {
		this.decisionCost = decisionCost;
	}

	public double getImportance() {
		return importance;
	}

	public void setImportance(double importance) {
		this.importance = importance;
	}

	/**
	 * Default constructor
	 */
	public Person(){
		
	}

	/**
	 * Main constructor used for a person
	 * @param community the community the person belongs to
	 * @param age initial age
	 * @param sex the sex
	 * @param location person's initial location
	 */
	public Person(Community community, int age,Sex sex,Point location){
		super();
		this.age=age;
		this.alive=true;
		this.sex=sex;
		this.id=Id;
		this.location=location;
		this.community=community;
		if(RunEnvironment.getInstance().isBatch())
			this.initV=false;
		Id++;
	}
	
	/**
	 * Method to move an agent (in animation mode) to a location
	 * @param newPoint
	 */
	public void moveTo(Point newPoint){
//		if(newPoint.getCentroid().getX()==community.getLocation().getCentroid().getX() &&
//			newPoint.getCentroid().getY()==community.getLocation().getCentroid().getY())
//			return;
		if(RunEnvironment.getInstance().isBatch())
			return;
		Context context = RunState.getInstance().getMasterContext();  
		Geography geography = (Geography)context.getProjection("Geography");
		this.location=newPoint;
//		geography.move(this,location);
		if(this.icon!=null){
	//		gis.getWwglCanvas().getModel().getLayers().remove(this.layer2);
	//		this.icon.setPosition(Position.fromDegrees(newPoint.getCentroid().getCoordinate().y, newPoint.getCentroid().getCoordinate().x,
	//				this.icon.getPosition().getElevation()));
	//		this.icon.move(this.icon.getPosition());
			this.icon.moveTo( new Position(Angle.fromDegreesLatitude(newPoint.getCoordinate().y), 
				Angle.fromDegreesLongitude(newPoint.getCoordinate().x), 0.0));
	//		this.newLayer(newPoint);
	//		gis.getWwglCanvas().redrawNow();
		}
		
	}
	


	public UserFacingIcon getIcon() {
		return icon;
	}

	public void setIcon(UserFacingIcon icon) {
		this.icon = icon;
	}

	@ScheduledMethod(start = 1, pick=ScheduledMethod.ALL, interval=1,priority = Double.MAX_VALUE-1)
	public void step(){
		if(field!=null){
			if(initV){
				initV(this.location);
				this.initV=false;
			}
			if(landUse==null)
				landUse=new LandUse(this,field);
			if(field.getId()>-1)
				landUse.step();
		}
	}
	
	/**
	 * Method for visualization of a person, used in animation mode
	 */
	public void initV(Point newPoint){
		List<IDisplay> displays=RunState.getInstance().getGUIRegistry().getDisplays();
		Iterator<IDisplay> ic = displays.iterator();
		this.gis=null;
		while(ic.hasNext()){
			IDisplay display = ic.next();
			if(display instanceof NewDisplayGIS)
				gis=(NewDisplayGIS)display;
		}
		Map<Class, AbstractSurfaceLayer> objs=gis.getClassStyleMap();
		AbstractSurfaceLayer layer = objs.get(Community.class);
		
		Collection<Renderable> renderables=(Collection<Renderable>)layer.getRenderables();
//		layer.getShapeToObjectMap().put(pS, this);
		
//		renderables.add(pS );
//		gis.getWwglCanvas().redraw();
		Context context = RunState.getInstance().getMasterContext();  
		Geography geography = (Geography)context.getProjection("Geography");
//		geography.move(this, location);
		
        this.layer2 = new IconLayer();
//      icon = new UserFacingIcon("src/images/32x32-icon-nasa.png",
        UserFacingIcon icon = new UserFacingIcon("../Salinization/img/People.png",
              new Position(Angle.fromDegreesLatitude(newPoint.getCentroid().getY()),
            		  Angle.fromDegrees(newPoint.getCentroid().getX()), 0));
        icon.setSize(new Dimension(14, 14));
        this.icon=icon;
        layer2.addIcon(icon);
       
		gis.getWwglCanvas().getModel().getLayers().add(layer2);
	}
	
	public void newLayer(Point newPoint){
		this.layer2 = new IconLayer();
//      icon = new UserFacingIcon("src/images/32x32-icon-nasa.png",
        UserFacingIcon icon = new UserFacingIcon("../Salinization/img/People.png",
              new Position(Angle.fromDegreesLatitude(newPoint.getCentroid().getY()),
            		  Angle.fromDegrees(newPoint.getCentroid().getX()), 0));
        icon.setSize(new Dimension(14, 14));
        this.icon=icon;
        layer2.addIcon(icon);
       
		gis.getWwglCanvas().getModel().getLayers().add(layer2);
	}
	
	/**
	 * Method for launching agent water use model (not currently used, as the behavior is done through the community)
	 */
//	@ScheduledMethod(start = 0, interval = 1, pick=ScheduledMethod.ALL, priority = 0)
	public void runWaterUseMode(){
//		waterUseModel.runModel();
	}
	
	public int getAge() {
		return age;
	}
	
	public void setAge(int age) {
		this.age = age;
	}
	public Sex getSex() {
		return sex;
	}
	public void setSex(Sex sex) {
		this.sex = sex;
	}
	

	public boolean isAlive() {
		return alive;
	}

	public void setAlive(boolean alive) {
		this.alive = alive;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public Community getCommunity() {
		return community;
	}

	public void setCommunity(Community community) {
		this.community = community;
	}


	public double getQuantityBelief() {
		return quantityBelief;
	}

	public void setQuantityBelief(double quantityBelief) {
		this.quantityBelief = quantityBelief;
	}

	public double getQuantityChangeBelief() {
		return quantityChangeBelief;	
	}

	public void setQuantityChangeBelief(double quantityChangeBelief) {
		this.quantityChangeBelief = quantityChangeBelief;
	}

	public double getSourcePreference() {
		return sourcePreference;
	}

	public void setSourcePreference(double sourcePreference) {
		this.sourcePreference = sourcePreference;
	}

	public double getQualityBelief() {
		return qualityBelief;
	}

	public void setQualityBelief(double qualityBelief) {
		this.qualityBelief = qualityBelief;
	}

	public double getQualityChangeBelief() {
		return qualityChangeBelief;
	}

	public void setQualityChangeBelief(double qualityChangeBelief) {
		this.qualityChangeBelief = qualityChangeBelief;
	}

	public double getOtherSourceQuantityBelief() {
		return otherSourceQuantityBelief;
	}

	public void setOtherSourceQuantityBelief(double otherSourceQuantityBelief) {
		this.otherSourceQuantityBelief = otherSourceQuantityBelief;
	}

	public double getOtherSourceQuantityChangeBelief() {
		return otherSourceQuantityChangeBelief;
	}

	public void setOtherSourceQuantityChangeBelief(
			double otherSourceQuantityChangeBelief) {
		this.otherSourceQuantityChangeBelief = otherSourceQuantityChangeBelief;
	}

	public double getOtherSourceQualityChangeBelief() {
		return otherSourceQualityChangeBelief;
	}

	public void setOtherSourceQualityChangeBelief(
			double otherSourceQualityChangeBelief) {
		this.otherSourceQualityChangeBelief = otherSourceQualityChangeBelief;
	}

	public double getOtherSourceQualityBelief() {
		return otherSourceQualityBelief;
	}

	public void setOtherSourceQualityBelief(double otherSourceQualityBelief) {
		this.otherSourceQualityBelief = otherSourceQualityBelief;
	}
	
	public double getX(){
		try{
			return location.getX();
		}
		catch(Exception e){
			return this.community.getLocation().getX();
		}
	}
	
	public double getY(){
		try{
			return location.getY();
		}
		catch(Exception e){
			return this.community.getLocation().getY();
		}
	}

	public boolean isAccuratePerception() {
		return accuratePerception;
	}

	public void setAccuratePerception(boolean accuratePerception) {
		this.accuratePerception = accuratePerception;
	}

	public double getWaterConsumed() {
		return waterConsumed;
	}

	public void setWaterConsumed(double waterConsumed) {
		this.waterConsumed = waterConsumed;
	}

	public int getSourceVisited() {
		return sourceVisited;
	}

	public void setSourceVisited(int sourceVisited) {
		this.sourceVisited = sourceVisited;
	}
	
	public Point getLocation() {
		return location;
	}

	public void setLocation(Point location) {
		this.location = location;
	}


	public void setX(double x) {
		this.x = x;
	}

	public void setY(double y) {
		this.y = y;
	}

	public double getMotivation() {
		return motivation;
	}

	public void setMotivation(double motivation) {
		this.motivation = motivation;
	}

	public int getAgree() {
		return agree;
	}

	public void setAgree(int agree) {
		this.agree = agree;
	}

	public Field getField() {
		return field;
	}

	public void setField(Field field) {
		this.field = field;
	}
	
	
	
}
